@@ -23,7 +23,7 @@
 
 #include "kformattest.h"
 
-#include <QtTest/QtTest>
+#include <QtTest>
 
 #include "kformat.h"
 
@@ -87,6 +87,37 @@ void KFormatTest::formatByteSize()
     // Spattering of specific units
     QCOMPARE(format.formatByteSize(823000, 3, KFormat::IECBinaryDialect, KFormat::UnitMegaByte), QStringLiteral("0.785 MiB"));
     QCOMPARE(format.formatByteSize(1234034.0, 4, KFormat::JEDECBinaryDialect, KFormat::UnitByte), QStringLiteral("1,234,034 B"));
+
+    // Check examples from the documentation
+    QCOMPARE(format.formatByteSize(1000, 1, KFormat::MetricBinaryDialect, KFormat::UnitKiloByte), QStringLiteral("1.0 kB"));
+    QCOMPARE(format.formatByteSize(1000, 1, KFormat::IECBinaryDialect, KFormat::UnitKiloByte), QStringLiteral("1.0 KiB"));
+    QCOMPARE(format.formatByteSize(1000, 1, KFormat::JEDECBinaryDialect, KFormat::UnitKiloByte), QStringLiteral("1.0 KB"));
+}
+
+void KFormatTest::formatValue()
+{
+    QLocale locale(QLocale::c());
+    locale.setNumberOptions(QLocale::DefaultNumberOptions); // Qt >= 5.6 sets QLocale::OmitGroupSeparator for the C locale
+    KFormat format(locale);
+
+    // Check examples from the documentation
+    QCOMPARE(format.formatValue(1000, KFormat::Unit::Byte, 1, KFormat::UnitPrefix::Kilo, KFormat::MetricBinaryDialect), QStringLiteral("1.0 kB"));
+    QCOMPARE(format.formatValue(1000, KFormat::Unit::Byte, 1, KFormat::UnitPrefix::Kilo, KFormat::IECBinaryDialect), QStringLiteral("1.0 KiB"));
+    QCOMPARE(format.formatValue(1000, KFormat::Unit::Byte, 1, KFormat::UnitPrefix::Kilo, KFormat::JEDECBinaryDialect), QStringLiteral("1.0 KB"));
+
+    // Check examples from the documentation
+    QCOMPARE(format.formatValue(1000, KFormat::Unit::Bit, 1, KFormat::UnitPrefix::Kilo, KFormat::MetricBinaryDialect), QStringLiteral("1.0 kbit"));
+    QCOMPARE(format.formatValue(1000, QStringLiteral("bit"), 1, KFormat::UnitPrefix::Kilo), QStringLiteral("1.0 kbit"));
+    QCOMPARE(format.formatValue(1000, QStringLiteral("bit/s"), 1, KFormat::UnitPrefix::Kilo), QStringLiteral("1.0 kbit/s"));
+
+    QCOMPARE(format.formatValue(100, QStringLiteral("bit/s")), QStringLiteral("100.0 bit/s"));
+    QCOMPARE(format.formatValue(1000, QStringLiteral("bit/s")), QStringLiteral("1.0 kbit/s"));
+    QCOMPARE(format.formatValue(10e3, QStringLiteral("bit/s")), QStringLiteral("10.0 kbit/s"));
+    QCOMPARE(format.formatValue(10e6, QStringLiteral("bit/s")), QStringLiteral("10.0 Mbit/s"));
+
+    QCOMPARE(format.formatValue(0.010, KFormat::Unit::Meter, 1, KFormat::UnitPrefix::Milli, KFormat::MetricBinaryDialect), QStringLiteral("10.0 mm"));
+    QCOMPARE(format.formatValue(10.12e-6, KFormat::Unit::Meter, 2, KFormat::UnitPrefix::Micro, KFormat::MetricBinaryDialect), QStringLiteral("10.12 µm"));
+    QCOMPARE(format.formatValue(10.55e-6, KFormat::Unit::Meter, 1, KFormat::UnitPrefix::AutoAdjust, KFormat::MetricBinaryDialect), QStringLiteral("10.6 µm"));
 }
 
 enum TimeConstants {
@@ -109,6 +140,7 @@ void KFormatTest::formatDuration()
     quint64 singleDay = 1 * MSecsInDay + 5 * MSecsInHour + 8 * MSecsInMinute + 3 * MSecsInSecond + 700;
     quint64 doubleDay = 10 * MSecsInDay + 5 * MSecsInHour + 8 * MSecsInMinute + 3 * MSecsInSecond + 700;
     quint64 roundingIssues = 2* MSecsInHour +  59 * MSecsInMinute + 59 * MSecsInSecond + 900;
+    quint64 largeValue = 9999999999;
 
     // Default format
     QCOMPARE(format.formatDuration(singleSecond), QStringLiteral("0:00:04"));
@@ -120,6 +152,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay), QStringLiteral("29:08:04"));
     QCOMPARE(format.formatDuration(doubleDay), QStringLiteral("245:08:04"));
     QCOMPARE(format.formatDuration(roundingIssues), QStringLiteral("3:00:00"));
+    QCOMPARE(format.formatDuration(largeValue), QStringLiteral("2777:46:40"));
 
 
     // ShowMilliseconds format
@@ -133,6 +166,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("29:08:03.700"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("245:08:03.700"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("2:59:59.900"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("2777:46:39.999"));
 
 
     // HideSeconds format
@@ -146,6 +180,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("29:08"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("245:08"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("3:00"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("2777:47"));
 
 
     // FoldHours format
@@ -159,6 +194,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("1748:04"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("14708:04"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("180:00"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("166666:40"));
 
 
     // FoldHours ShowMilliseconds format
@@ -173,6 +209,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("1748:03.700"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("14708:03.700"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("179:59.900"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("166666:39.999"));
 
 
     // InitialDuration format
@@ -186,6 +223,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("29h08m04s"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("245h08m04s"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("3h00m00s"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("2777h46m40s"));
 
 
     // InitialDuration and ShowMilliseconds format
@@ -200,6 +238,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("29h08m03.700s"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("245h08m03.700s"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("2h59m59.900s"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("2777h46m39.999s"));
 
 
     // InitialDuration and HideSeconds format
@@ -214,6 +253,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("29h08m"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("245h08m"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("3h00m"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("2777h47m"));
 
 
     // InitialDuration and FoldHours format
@@ -228,6 +268,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("1748m04s"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("14708m04s"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("180m00s"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("166666m40s"));
 
 
     // InitialDuration and FoldHours and ShowMilliseconds format
@@ -242,6 +283,7 @@ void KFormatTest::formatDuration()
     QCOMPARE(format.formatDuration(singleDay, options), QStringLiteral("1748m03.700s"));
     QCOMPARE(format.formatDuration(doubleDay, options), QStringLiteral("14708m03.700s"));
     QCOMPARE(format.formatDuration(roundingIssues, options), QStringLiteral("179m59.900s"));
+    QCOMPARE(format.formatDuration(largeValue,options), QStringLiteral("166666m39.999s"));
 }
 
 void KFormatTest::formatDecimalDuration()
