@@ -29,7 +29,7 @@
 // Jul 30, 2007 - Substituted addEntry boolean params with KDirWatch::WatchModes
 // flag (Flavio Castelli)
 // Oct 4,  2005 - Inotify support (Dirk Mueller)
-// Februar 2002 - Add file watching and remote mount check for STAT
+// February 2002 - Add file watching and remote mount check for STAT
 // Mar 30, 2001 - Native support for Linux dir change notification.
 // Jan 28, 2000 - Usage of FAM service on IRIX (Josef.Weidendorfer@in.tum.de)
 // May 24. 1998 - List of times introduced, and some bugs are fixed. (sven)
@@ -51,13 +51,13 @@
 #include <assert.h>
 #include <errno.h>
 #include <QLoggingCategory>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QSocketNotifier>
-#include <QtCore/QTimer>
-#include <QtCore/QThread>
-#include <QtCore/QThreadStorage>
-#include <QtCore/QCoreApplication>
+#include <QDir>
+#include <QFile>
+#include <QSocketNotifier>
+#include <QTimer>
+#include <QThread>
+#include <QThreadStorage>
+#include <QCoreApplication>
 
 #include <qplatformdefs.h> // QT_LSTAT, QT_STAT, QT_STATBUF
 
@@ -158,8 +158,8 @@ static const char s_envNfsMethod[] = "KDIRWATCH_NFSMETHOD";
  * - FAM (File Alternation Monitor): first used on IRIX, SGI
  *   has ported this method to LINUX. It uses a kernel part
  *   (IMON, sending change events to /dev/imon) and a user
- *   level damon (fam), to which applications connect for
- *   notification of file changes. For NFS, the fam damon
+ *   level daemon (fam), to which applications connect for
+ *   notification of file changes. For NFS, the fam daemon
  *   on the NFS server machine is used; if IMON is not built
  *   into the kernel, fam uses polling for local files.
  * - INOTIFY: In LINUX 2.6.13, inode change notification was
@@ -187,8 +187,8 @@ KDirWatchPrivate::KDirWatchPrivate()
     timer.setObjectName(QStringLiteral("KDirWatchPrivate::timer"));
     connect(&timer, SIGNAL(timeout()), this, SLOT(slotRescan()));
 
-    m_nfsPollInterval = qEnvironmentVariableIsSet(s_envNfsPoll) ? qgetenv(s_envNfsPoll).toInt() : 5000;
-    m_PollInterval = qEnvironmentVariableIsSet(s_envPoll) ? qgetenv(s_envPoll).toInt() : 500;
+    m_nfsPollInterval = qEnvironmentVariableIsSet(s_envNfsPoll) ? qEnvironmentVariableIntValue(s_envNfsPoll) : 5000;
+    m_PollInterval = qEnvironmentVariableIsSet(s_envPoll) ? qEnvironmentVariableIntValue(s_envPoll) : 500;
 
     m_preferredMethod = methodFromString(qEnvironmentVariableIsSet(s_envMethod) ? qgetenv(s_envMethod) : "inotify");
     // The nfs method defaults to the normal (local) method
@@ -206,7 +206,7 @@ KDirWatchPrivate::KDirWatchPrivate()
 #if HAVE_FAM
     availableMethods << "FAM";
     use_fam = true;
-    sn = 0;
+    sn = nullptr;
 #endif
 
 #if HAVE_SYS_INOTIFY_H
@@ -312,132 +312,124 @@ void KDirWatchPrivate::inotifyEventReceived()
             // Is set to true if the new event is a directory, false otherwise. This prevents a stat call in clientsForFileOrDir
             const bool isDir = (event->mask & (IN_ISDIR));
 
-            // now we're in deep trouble of finding the
-            // associated entries
-            // for now, we suck and iterate
-            for (EntryMap::Iterator it = m_mapEntries.begin();
-                    it != m_mapEntries.end();) {
+            Entry *e = m_inotify_wd_to_entry.value(event->wd);
+            if (!e) {
+                continue;
+            }
+            const bool wasDirty = e->dirty;
+            e->dirty = true;
 
-                Entry *e = &(*it);
-                ++it;
-                if (e->wd == event->wd) {
-                    const bool wasDirty = e->dirty;
-                    e->dirty = true;
+            const QString tpath = e->path + QLatin1Char('/') + path;
 
-                    const QString tpath = e->path + QLatin1Char('/') + path;
+            if (s_verboseDebug) {
+                qCDebug(KDIRWATCH).nospace() << "got event 0x" << qPrintable(QString::number(event->mask, 16)) << " for " << e->path;
+            }
 
-                    if (s_verboseDebug) {
-                      qCDebug(KDIRWATCH).nospace() << "got event 0x" << qPrintable(QString::number(event->mask, 16)) << " for " << e->path;
-                    }
+            if (event->mask & IN_DELETE_SELF) {
+                if (s_verboseDebug) {
+                    qCDebug(KDIRWATCH) << "-->got deleteself signal for" << e->path;
+                }
+                e->m_status = NonExistent;
+                m_inotify_wd_to_entry.remove(e->wd);
+                e->wd = -1;
+                e->m_ctime = invalid_ctime;
+                emitEvent(e, Deleted, e->path);
+                // If the parent dir was already watched, tell it something changed
+                Entry *parentEntry = entry(e->parentDirectory());
+                if (parentEntry) {
+                    parentEntry->dirty = true;
+                }
+                // Add entry to parent dir to notice if the entry gets recreated
+                addEntry(nullptr, e->parentDirectory(), e, true /*isDir*/);
+            }
+            if (event->mask & IN_IGNORED) {
+                // Causes bug #207361 with kernels 2.6.31 and 2.6.32!
+                //e->wd = -1;
+            }
+            if (event->mask & (IN_CREATE | IN_MOVED_TO)) {
+                Entry *sub_entry = e->findSubEntry(tpath);
 
-                    if (event->mask & IN_DELETE_SELF) {
-                        if (s_verboseDebug) {
-                            qCDebug(KDIRWATCH) << "-->got deleteself signal for" << e->path;
-                        }
-                        e->m_status = NonExistent;
-                        e->wd = -1;
-                        e->m_ctime = invalid_ctime;
-                        emitEvent(e, Deleted, e->path);
-                        // If the parent dir was already watched, tell it something changed
-                        Entry *parentEntry = entry(e->parentDirectory());
-                        if (parentEntry) {
-                            parentEntry->dirty = true;
-                        }
-                        // Add entry to parent dir to notice if the entry gets recreated
-                        addEntry(nullptr, e->parentDirectory(), e, true /*isDir*/);
-                    }
-                    if (event->mask & IN_IGNORED) {
-                        // Causes bug #207361 with kernels 2.6.31 and 2.6.32!
-                        //e->wd = -1;
-                    }
-                    if (event->mask & (IN_CREATE | IN_MOVED_TO)) {
-                        Entry *sub_entry = e->findSubEntry(tpath);
+                if (s_verboseDebug) {
+                    qCDebug(KDIRWATCH) << "-->got CREATE signal for" << (tpath) << "sub_entry=" << sub_entry;
+                    qCDebug(KDIRWATCH) << *e;
+                }
 
-                        if (s_verboseDebug) {
-                            qCDebug(KDIRWATCH) << "-->got CREATE signal for" << (tpath) << "sub_entry=" << sub_entry;
-                            qCDebug(KDIRWATCH) << *e;
-                        }
-
-                        // The code below is very similar to the one in checkFAMEvent...
-                        if (sub_entry) {
-                            // We were waiting for this new file/dir to be created
-                            sub_entry->dirty = true;
-                            rescan_timer.start(0); // process this asap, to start watching that dir
-                        } else if (e->isDir && !e->m_clients.empty()) {
-                            const QList<const Client *> clients = e->inotifyClientsForFileOrDir(isDir);
-                            // See discussion in addEntry for why we don't addEntry for individual
-                            // files in WatchFiles mode with inotify.
-                            if (isDir) {
-                                for (const Client *client : clients) {
-                                    addEntry(client->instance, tpath, nullptr, isDir,
-                                             isDir ? client->m_watchModes : KDirWatch::WatchDirOnly);
-                                }
-                            }
-                            if (!clients.isEmpty()) {
-                                emitEvent(e, Created, tpath);
-                                qCDebug(KDIRWATCH).nospace() << clients.count() << " instance(s) monitoring the new "
-                                                   << (isDir ? "dir " : "file ") << tpath;
-                            }
-                            e->m_pendingFileChanges.append(e->path);
-                            if (!rescan_timer.isActive()) {
-                                rescan_timer.start(m_PollInterval);    // singleshot
-                            }
+                // The code below is very similar to the one in checkFAMEvent...
+                if (sub_entry) {
+                    // We were waiting for this new file/dir to be created
+                    sub_entry->dirty = true;
+                    rescan_timer.start(0); // process this asap, to start watching that dir
+                } else if (e->isDir && !e->m_clients.empty()) {
+                    const QList<const Client *> clients = e->inotifyClientsForFileOrDir(isDir);
+                    // See discussion in addEntry for why we don't addEntry for individual
+                    // files in WatchFiles mode with inotify.
+                    if (isDir) {
+                        for (const Client *client : clients) {
+                            addEntry(client->instance, tpath, nullptr, isDir,
+                                        isDir ? client->m_watchModes : KDirWatch::WatchDirOnly);
                         }
                     }
-                    if (event->mask & (IN_DELETE | IN_MOVED_FROM)) {
-                        if (s_verboseDebug) {
-                            qCDebug(KDIRWATCH) << "-->got DELETE signal for" << tpath;
-                        }
-                        if ((e->isDir) && (!e->m_clients.empty())) {
-                            // A file in this directory has been removed.  It wasn't an explicitly
-                            // watched file as it would have its own watch descriptor, so
-                            // no addEntry/ removeEntry bookkeeping should be required.  Emit
-                            // the event immediately if any clients are interested.
-                            KDirWatch::WatchModes flag = isDir ? KDirWatch::WatchSubDirs : KDirWatch::WatchFiles;
-                            int counter = 0;
-                            for (const Client &client : e->m_clients) {
-                                if (client.m_watchModes & flag) {
-                                    counter++;
-                                }
-                            }
-                            if (counter != 0) {
-                                emitEvent(e, Deleted, tpath);
-                            }
-                        }
+                    if (!clients.isEmpty()) {
+                        emitEvent(e, Created, tpath);
+                        qCDebug(KDIRWATCH).nospace() << clients.count() << " instance(s) monitoring the new "
+                                            << (isDir ? "dir " : "file ") << tpath;
                     }
-                    if (event->mask & (IN_MODIFY | IN_ATTRIB)) {
-                        if ((e->isDir) && (!e->m_clients.empty())) {
-                            if (s_verboseDebug) {
-                                qCDebug(KDIRWATCH) << "-->got MODIFY signal for" << (tpath);
-                            }
-                            // A file in this directory has been changed.  No
-                            // addEntry/ removeEntry bookkeeping should be required.
-                            // Add the path to the list of pending file changes if
-                            // there are any interested clients.
-                            //QT_STATBUF stat_buf;
-                            //QByteArray tpath = QFile::encodeName(e->path+'/'+path);
-                            //QT_STAT(tpath, &stat_buf);
-                            //bool isDir = S_ISDIR(stat_buf.st_mode);
-
-                            // The API doc is somewhat vague as to whether we should emit
-                            // dirty() for implicitly watched files when WatchFiles has
-                            // not been specified - we'll assume they are always interested,
-                            // regardless.
-                            // Don't worry about duplicates for the time
-                            // being; this is handled in slotRescan.
-                            e->m_pendingFileChanges.append(tpath);
-                            // Avoid stat'ing the directory if only an entry inside it changed.
-                            e->dirty = (wasDirty || (path.isEmpty() && (event->mask & IN_ATTRIB)));
-                        }
-                    }
-
+                    e->m_pendingFileChanges.append(e->path);
                     if (!rescan_timer.isActive()) {
                         rescan_timer.start(m_PollInterval);    // singleshot
                     }
-
-                    break;
                 }
             }
+            if (event->mask & (IN_DELETE | IN_MOVED_FROM)) {
+                if (s_verboseDebug) {
+                    qCDebug(KDIRWATCH) << "-->got DELETE signal for" << tpath;
+                }
+                if ((e->isDir) && (!e->m_clients.empty())) {
+                    // A file in this directory has been removed.  It wasn't an explicitly
+                    // watched file as it would have its own watch descriptor, so
+                    // no addEntry/ removeEntry bookkeeping should be required.  Emit
+                    // the event immediately if any clients are interested.
+                    KDirWatch::WatchModes flag = isDir ? KDirWatch::WatchSubDirs : KDirWatch::WatchFiles;
+                    int counter = 0;
+                    for (const Client &client : e->m_clients) {
+                        if (client.m_watchModes & flag) {
+                            counter++;
+                        }
+                    }
+                    if (counter != 0) {
+                        emitEvent(e, Deleted, tpath);
+                    }
+                }
+            }
+            if (event->mask & (IN_MODIFY | IN_ATTRIB)) {
+                if ((e->isDir) && (!e->m_clients.empty())) {
+                    if (s_verboseDebug) {
+                        qCDebug(KDIRWATCH) << "-->got MODIFY signal for" << (tpath);
+                    }
+                    // A file in this directory has been changed.  No
+                    // addEntry/ removeEntry bookkeeping should be required.
+                    // Add the path to the list of pending file changes if
+                    // there are any interested clients.
+                    //QT_STATBUF stat_buf;
+                    //QByteArray tpath = QFile::encodeName(e->path+'/'+path);
+                    //QT_STAT(tpath, &stat_buf);
+                    //bool isDir = S_ISDIR(stat_buf.st_mode);
+
+                    // The API doc is somewhat vague as to whether we should emit
+                    // dirty() for implicitly watched files when WatchFiles has
+                    // not been specified - we'll assume they are always interested,
+                    // regardless.
+                    // Don't worry about duplicates for the time
+                    // being; this is handled in slotRescan.
+                    e->m_pendingFileChanges.append(tpath);
+                    // Avoid stat'ing the directory if only an entry inside it changed.
+                    e->dirty = (wasDirty || (path.isEmpty() && (event->mask & IN_ATTRIB)));
+                }
+            }
+
+            if (!rescan_timer.isActive()) {
+                rescan_timer.start(m_PollInterval);    // singleshot
+            }
         }
         if (bytesAvailable > 0) {
             // copy partial event to beginning of buffer
@@ -650,7 +642,7 @@ bool KDirWatchPrivate::useFAM(Entry *e)
     if (e->isDir) {
         if (e->m_status == NonExistent) {
             // If the directory does not exist we watch the parent directory
-            addEntry(0, e->parentDirectory(), e, true);
+            addEntry(nullptr, e->parentDirectory(), e, true);
         } else {
             int res = FAMMonitorDirectory(&fc, QFile::encodeName(e->path).data(),
                                           &(e->fr), e);
@@ -659,7 +651,7 @@ bool KDirWatchPrivate::useFAM(Entry *e)
                 e->m_mode = UnknownMode;
                 use_fam = false;
                 delete sn;
-                sn = 0;
+                sn = nullptr;
                 return false;
             }
             qCDebug(KDIRWATCH).nospace() << " Setup FAM (Req " << FAMREQUEST_GETREQNUM(&(e->fr))
@@ -668,7 +660,7 @@ bool KDirWatchPrivate::useFAM(Entry *e)
     } else {
         if (e->m_status == NonExistent) {
             // If the file does not exist we watch the directory
-            addEntry(0, QFileInfo(e->path).absolutePath(), e, true);
+            addEntry(nullptr, QFileInfo(e->path).absolutePath(), e, true);
         } else {
             int res = FAMMonitorFile(&fc, QFile::encodeName(e->path).data(),
                                      &(e->fr), e);
@@ -677,7 +669,7 @@ bool KDirWatchPrivate::useFAM(Entry *e)
                 e->m_mode = UnknownMode;
                 use_fam = false;
                 delete sn;
-                sn = 0;
+                sn = nullptr;
                 return false;
             }
 
@@ -726,6 +718,7 @@ bool KDirWatchPrivate::useINotify(Entry *e)
 
     if ((e->wd = inotify_add_watch(m_inotify_fd,
                                    QFile::encodeName(e->path).data(), mask)) >= 0) {
+        m_inotify_wd_to_entry.insert(e->wd, e);
         if (s_verboseDebug) {
             qCDebug(KDIRWATCH) << "inotify successfully used for monitoring" << e->path << "wd=" << e->wd;
         }
@@ -1004,6 +997,7 @@ void KDirWatchPrivate::removeWatch(Entry *e)
 #endif
 #if HAVE_SYS_INOTIFY_H
     if (e->m_mode == INotifyMode) {
+        m_inotify_wd_to_entry.remove(e->wd);
         (void) inotify_rm_watch(m_inotify_fd, e->wd);
         if (s_verboseDebug) {
             qCDebug(KDIRWATCH).nospace() << "Cancelled INotify (fd " << m_inotify_fd << ", "
@@ -1081,6 +1075,9 @@ void KDirWatchPrivate::removeEntry(KDirWatch *instance,
                            << " [" << (instance ? instance->objectName() : QString()) << "]";
     }
     QString p = e->path; // take a copy, QMap::remove takes a reference and deletes, since e points into the map
+#if HAVE_SYS_INOTIFY_H
+    m_inotify_wd_to_entry.remove(e->wd);
+#endif
     m_mapEntries.remove(p); // <e> not valid any more
 }
 
@@ -1402,19 +1399,31 @@ void KDirWatchPrivate::emitEvent(Entry *e, int event, const QString &fileName)
             continue;
         }
 
-        // Emit the signals delayed, to avoid unexpected re-entrancy from the slots (#220153)
+        // Emit the signals delayed, to avoid unexpected re-entrance from the slots (#220153)
 
         if (event & Deleted) {
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
             QMetaObject::invokeMethod(c.instance, "setDeleted", Qt::QueuedConnection, Q_ARG(QString, path));
+#else
+            QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setDeleted(path); }, Qt::QueuedConnection);
+#endif
         }
 
         if (event & Created) {
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
             QMetaObject::invokeMethod(c.instance, "setCreated", Qt::QueuedConnection, Q_ARG(QString, path));
+#else
+            QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setCreated(path); }, Qt::QueuedConnection);
+#endif
             // possible emit Change event after creation
         }
 
         if (event & Changed) {
+#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
             QMetaObject::invokeMethod(c.instance, "setDirty", Qt::QueuedConnection, Q_ARG(QString, path));
+#else
+            QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setDirty(path); }, Qt::QueuedConnection);
+#endif
         }
     }
 }
@@ -1465,7 +1474,7 @@ void KDirWatchPrivate::slotRescan()
         }
         rescan_all = false;
     } else {
-        // progate dirty flag to dependant entries (e.g. file watches)
+        // propagate dirty flag to dependent entries (e.g. file watches)
         it = m_mapEntries.begin();
         for (; it != m_mapEntries.end(); ++it)
             if (((*it).m_mode == INotifyMode || (*it).m_mode == QFSWatchMode) && (*it).dirty) {
@@ -1522,7 +1531,7 @@ void KDirWatchPrivate::slotRescan()
 
 #if HAVE_SYS_INOTIFY_H
         if (entry->isDir) {
-            // Report and clear the the list of files that have changed in this directory.
+            // Report and clear the list of files that have changed in this directory.
             // Remove duplicates by changing to set and back again:
             // we don't really care about preserving the order of the
             // original changes.
@@ -1590,7 +1599,7 @@ void KDirWatchPrivate::famEventReceived()
         if (FAMNextEvent(&fc, &fe) == -1) {
             qCWarning(KCOREADDONS_DEBUG) << "FAM connection problem, switching to polling.";
             use_fam = false;
-            delete sn; sn = 0;
+            delete sn; sn = nullptr;
 
             // Replace all FAMMode entries with INotify/Stat
             EntryMap::Iterator it = m_mapEntries.begin();
@@ -1611,7 +1620,7 @@ void KDirWatchPrivate::checkFAMEvent(FAMEvent *fe)
 {
     //qCDebug(KDIRWATCH);
 
-    Entry *e = 0;
+    Entry *e = nullptr;
     EntryMap::Iterator it = m_mapEntries.begin();
     for (; it != m_mapEntries.end(); ++it)
         if (FAMREQUEST_GETREQNUM(&((*it).fr)) ==
@@ -1686,7 +1695,7 @@ void KDirWatchPrivate::checkFAMEvent(FAMEvent *fe)
                 parentEntry->dirty = true;
             }
             // Add entry to parent dir to notice if the entry gets recreated
-            addEntry(0, e->parentDirectory(), e, true /*isDir*/);
+            addEntry(nullptr, e->parentDirectory(), e, true /*isDir*/);
         } else {
             // A file in this directory has been removed, and wasn't explicitly watched.
             // We could still inform clients, like inotify does? But stat can't.
@@ -1711,7 +1720,7 @@ void KDirWatchPrivate::checkFAMEvent(FAMEvent *fe)
             bool isDir = false;
             const QList<const Client *> clients = e->clientsForFileOrDir(tpath, &isDir);
             Q_FOREACH (const Client *client, clients) {
-                addEntry(client->instance, tpath, 0, isDir,
+                addEntry(client->instance, tpath, nullptr, isDir,
                          isDir ? client->m_watchModes : KDirWatch::WatchDirOnly);
             }
 
@@ -1768,7 +1777,7 @@ void KDirWatchPrivate::statistics()
                 qCDebug(KDIRWATCH)  << "    by " << c.instance->objectName()
                           << " (" << c.count << " times)" << pending;
             }
-            if (e->m_entries.count() > 0) {
+            if (!e->m_entries.isEmpty()) {
                 qCDebug(KDIRWATCH) << "    dependent entries:";
                 Q_FOREACH (Entry *d, e->m_entries) {
                     qCDebug(KDIRWATCH) << "      " << d << d->path << (d->m_status == NonExistent ? "NonExistent" : "EXISTS!!! ERROR!");
