@@ -69,12 +69,12 @@ typedef std::unique_ptr<std::remove_pointer<HANDLE>::type, decltype(handleCloser
 */
 template<typename T> struct NetApiTypeInfo {};
 #define NETAPI_TYPE_INFO(prefix, n) template<> struct NetApiTypeInfo<prefix##_##n> { enum { level = n }; };
-NETAPI_TYPE_INFO(GROUP_INFO, 0);
-NETAPI_TYPE_INFO(GROUP_INFO, 3);
-NETAPI_TYPE_INFO(USER_INFO, 0);
-NETAPI_TYPE_INFO(USER_INFO, 4);
-NETAPI_TYPE_INFO(USER_INFO, 11);
-NETAPI_TYPE_INFO(GROUP_USERS_INFO, 0);
+NETAPI_TYPE_INFO(GROUP_INFO, 0)
+NETAPI_TYPE_INFO(GROUP_INFO, 3)
+NETAPI_TYPE_INFO(USER_INFO, 0)
+NETAPI_TYPE_INFO(USER_INFO, 4)
+NETAPI_TYPE_INFO(USER_INFO, 11)
+NETAPI_TYPE_INFO(GROUP_USERS_INFO, 0)
 
 // T must be a USER_INFO_* structure
 template<typename T>
@@ -154,7 +154,7 @@ void enumerateGroupsForUser(uint maxCount, const QString &name, Callback callbac
     netApiEnumerate<T>(maxCount, callback, [&](int level, LPBYTE *buffer, DWORD *count, DWORD *total, PDWORD_PTR resumeHandle) -> NET_API_STATUS {
         Q_UNUSED(resumeHandle);
         NET_API_STATUS ret = NetUserGetGroups(nullptr, nameStr, level, buffer, MAX_PREFERRED_LENGTH, count, total);
-        // if we return ERROR_MORE_DATA here it will result in an enless loop
+        // if we return ERROR_MORE_DATA here it will result in an endless loop
         if (ret == ERROR_MORE_DATA) {
             qCWarning(KCOREADDONS_DEBUG) << "NetUserGetGroups for user" << name << "returned ERROR_MORE_DATA. This should not happen!";
             ret = NERR_Success;
@@ -172,7 +172,7 @@ void enumerateUsersForGroup(const QString &name, uint maxCount, Callback callbac
     });
 }
 
-class KUser::Private : public QSharedData
+class Q_DECL_HIDDEN KUser::Private : public QSharedData
 {
     typedef QExplicitlySharedDataPointer<Private> Ptr;
     Private() : isAdmin(false) {}
@@ -486,7 +486,7 @@ KUser::~KUser()
 {
 }
 
-class KUserGroup::Private : public QSharedData
+class Q_DECL_HIDDEN KUserGroup::Private : public QSharedData
 {
 public:
     QString name;
@@ -853,7 +853,7 @@ static std::unique_ptr<char[]> queryProcessInformation(TOKEN_INFORMATION_CLASS t
     HANDLE _token;
     if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &_token)) {
         qWarning("Failed to get the token for the current process: %d", (int)GetLastError());
-        return false;
+        return nullptr;
     }
     ScopedHANDLE token(_token, handleCloser);
     // query required size
